package com.mediahub

import akka.{Done, NotUsed}
import org.springframework.stereotype.Component

import java.io.File
import akka.actor.ActorSystem
import akka.stream.alpakka.csv.scaladsl.{CsvParsing, CsvToMap}
import akka.stream.javadsl.FileIO
import akka.stream.scaladsl.{Compression, Flow, Framing, Keep, Sink, Source}
import akka.util.ByteString

import java.nio.file.Paths
import java.time.Duration
import java.util.stream.Collectors
import scala.collection.immutable
import scala.concurrent.ExecutionContext.Implicits.global
import scala.concurrent.duration.DurationInt
import scala.concurrent.{Await, ExecutionContext, Future}
import scala.language.postfixOps
import scala.util.{Failure, Success}


@Component
class FileReader {

  implicit val system: ActorSystem = ActorSystem()

  val list: List[File] = getListOfFiles
  val source: Source[File, NotUsed] = Source(list)

  val result: Future[immutable.Iterable[Option[String]]] = source
    .flatMapConcat(f => FileIO.fromPath(Paths.get(f.getPath), 1 * 1024 * 1024))
    .via(CsvParsing.lineScanner(CsvParsing.Tab))
    .via(CsvToMap.toMapAsStringsCombineAll(headerPlaceholder = Option.empty))
    .filter(row => row.getOrElse("primaryTitle", "") == "Autour d'une cabine")
    .map(a => a.get("tconst"))
    .runWith(Sink.collection)

  result.onComplete {
    case Failure(exception) => println(s"failure: $exception")

    case Success(value) => {
      println(s"sucess: $value")
    }

  }
  Await.result(result,  5 hours)

  val re: List[Option[String]] = result.value.get.get.toList

  println("value ", re.iterator.next().get)

  //.runWith(Sink.seq)
/*
  source
    .flatMapConcat(f => FileIO.fromPath(Paths.get(f.getPath), 1 * 1024 * 1024))
    .via(CsvParsing.lineScanner(CsvParsing.Tab))
    .via(CsvToMap.toMapAsStringsCombineAll(headerPlaceholder = Option.empty))
    .runWith(Sink.collection)
    .foreach(a => println("result", a))
*/

  /* .filter(row => row.getOrElse("tconst", "name") == (result.value, "Antoine"))
   .runForeach(println)*/


  // readAsync()


  /* def readAsync(): Source = {
     Source(list)
       .mapAsync(4) { value =>
         implicit val ec: ExecutionContext = ActorSystem().dispatcher
         Future {
           source
             .flatMapConcat(f => FileIO.fromPath(Paths.get(f.getPath), 1 * 1024 * 1024))
             .via(CsvParsing.lineScanner(CsvParsing.Tab))
             .via(CsvToMap.toMapAsStringsCombineAll(headerPlaceholder = Option.empty))
             .filter(row => row.getOrElse("primaryTitle", "") == "Autour d'une cabine")
             .runForeach(println)
           println(s"Process in Thread:${Thread.currentThread().getName}")
           value
         }
       }
       .runWith(Sink.foreach(value => println(s"Item emitted:$value in Thread:${Thread.currentThread().getName}")))
   }*/

  def getListOfFiles: List[File] = {
    val d = new File("src/main/resources")
    if (d.exists && d.isDirectory) {
      d.listFiles.filter(_.isFile).toList
    } else {
      List[File]()
    }
  }

}
